//#include "config.h"

#include "hildon-window.h"

typedef struct {
  GtkWidget *header;
  GtkWidget *window_stack;
  GtkWidget *back_button;
} HildonWindowPrivate;

enum {
  PROP_0,
  NUM_PROPERTIES
};

static GParamSpec *props[NUM_PROPERTIES] = { NULL, };

G_DEFINE_TYPE_WITH_PRIVATE (HildonWindow, hildon_window, GTK_TYPE_WINDOW)


// These two functions handle adding/removing widgets anywhere in the stack.
static void
on_stack_child_added (GtkContainer *container, GtkWidget *widget, HildonWindow *window)
{
  HildonWindowPrivate *priv = hildon_window_get_instance_private (window);
  GList* children = gtk_container_get_children (container);
  guint count = g_list_length (children);
  g_list_free (children);
  
  if (count == 2)
  {
    g_warning ("Show back");
    gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->header), priv->back_button);
    gtk_widget_set_visible (priv->back_button, TRUE);
    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), FALSE);
  }
  else
    g_warning ("Show close");
}

static void
on_stack_child_removed (GtkContainer *container, GtkWidget *widget, HildonWindow *window)
{
  HildonWindowPrivate *priv = hildon_window_get_instance_private (window);
  GList* children = gtk_container_get_children (container);
  guint count = g_list_length (children);
  g_list_free (children);
  
  if (count > 1)
  {
    g_warning ("Show back");
  }
  else if (count == 1)
  {
    g_warning ("Show close");
    gtk_widget_set_visible (priv->back_button, FALSE);
    g_object_ref (priv->back_button); // Prevent error when adding new pages.
    gtk_container_remove (GTK_CONTAINER (priv->header), priv->back_button);
    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), TRUE);
  }
}

static void
hildon_window_class_init (HildonWindowClass *class)
{
  GObjectClass *object_class = G_OBJECT_CLASS (class);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
  GtkContainerClass *container_class = GTK_CONTAINER_CLASS (class);
}

/*
 * TEST ONLY
 *
 * Conditions for test:
 * Presume all pages are visible, we can extend to check visibility later.
 *
 * Adding items isn't really an issue. Removing them is.  Going forward, the
 * remove functionality here would probably have to actual be part of HildonWindow.
 * 
 * Some of this code is a bit messy so that we can use the window manager provided
 * close icon.  We could just change the button callback and icon at runtime, making
 * use of window-close or a custom icon.  It would make the code cleaner but what
 * about a desktop case mixing Hildon and non-Hildon apps?
 *
 */
static void
add_clicked (GtkWidget *widget, GtkStack *stack)
{
  GtkWidget* child = gtk_label_new ("I'm a child");
  gtk_container_add (GTK_CONTAINER (stack), child);
  gint position;
  gtk_container_child_get (GTK_CONTAINER (stack), child, "position", &position, NULL);

  gtk_widget_set_visible (child, TRUE);
  g_warning ("Added child @ position %i", position);
  gtk_stack_set_visible_child (stack, child);
}

static void
remove_clicked (GtkWidget *widget, HildonWindow *window)
{
  HildonWindowPrivate *priv = hildon_window_get_instance_private (window);
  GtkStack *stack = GTK_STACK (priv->window_stack);
  GtkWidget *child = gtk_stack_get_visible_child (stack);
  if (child)
  {
    gint position;
    gtk_container_child_get (GTK_CONTAINER (stack), child, "position", &position, NULL);
    
    GList *children = gtk_container_get_children (GTK_CONTAINER (stack));
    GList *pos = g_list_find (children, child);
    g_assert (pos != NULL);
    if (pos->prev) // Needed to avoid navigating to non existant.
    {
      // Removing child before changing visible item has unusual results.
      // Popping last item will work ONCE. Popping again shows first page.
      gtk_stack_set_visible_child (stack, pos->prev->data);
      gtk_container_remove (GTK_CONTAINER (stack), child);
      g_warning ("Removed child @ position %i", position);
      if (!pos->prev->prev)  // If we are now the last page, show close button
      {
        gtk_widget_set_visible (priv->back_button, FALSE);
        gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), TRUE);
      }
    }
    else
    {
      g_warning ("No other children to remove/ on first page");  // We were on first/only page.  Close normally
    }
    g_list_free (children);
  }
  else
  {
    g_warning ("No child to remove"); // Normal non-stacked window? Close normally
    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), TRUE);
  }
}

static void
hildon_window_init (HildonWindow *window)
{
  HildonWindowPrivate *priv = hildon_window_get_instance_private (window);
  
  priv->header = gtk_header_bar_new ();
  gtk_window_set_titlebar (GTK_WINDOW (window), priv->header);
  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->header), TRUE);
  gtk_header_bar_set_title (GTK_HEADER_BAR (priv->header), "Hildon UI Concept");
  gtk_header_bar_set_subtitle (GTK_HEADER_BAR (priv->header), "Pure GTK3");
  gtk_header_bar_set_decoration_layout (GTK_HEADER_BAR (priv->header), ":close");
  
  priv->window_stack = gtk_stack_new ();
  gtk_stack_set_transition_type (GTK_STACK (priv->window_stack), GTK_STACK_TRANSITION_TYPE_SLIDE_LEFT_RIGHT);
  gtk_stack_set_transition_duration (GTK_STACK (priv->window_stack), 500);
  gtk_container_add (GTK_CONTAINER (window), priv->window_stack);
  g_signal_connect_after (priv->window_stack, "add", G_CALLBACK (on_stack_child_added), window);
  g_signal_connect_after (priv->window_stack, "remove", G_CALLBACK (on_stack_child_removed), window);
  
  //TEST ONLY
  priv->back_button = gtk_button_new_from_icon_name ("go-previous-symbolic", GTK_ICON_SIZE_MENU);
  gtk_button_set_relief (GTK_BUTTON (priv->back_button), GTK_RELIEF_NONE);
  g_signal_connect (priv->back_button, "clicked", G_CALLBACK (remove_clicked), window);
  gtk_widget_set_visible (priv->back_button, FALSE);
  
  GtkWidget *widget = gtk_button_new_from_icon_name ("list-add", GTK_ICON_SIZE_MENU);
  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->header), widget);
  g_signal_connect (widget, "clicked", G_CALLBACK (add_clicked), priv->window_stack);
}

/**
 * hildon_window_new: 
 * 
 * Creates a new #HildonWindow.
 * 
 * Return value: A #HildonWindow.
 **/
GtkWidget*
hildon_window_new (void)
{
    HildonWindow *window = g_object_new (HILDON_TYPE_WINDOW, NULL);

    return GTK_WIDGET (window);
}


